# Install Concept products from local distr

# Setup constants
Set-Variable -Name OFFICE_REGISTRY -Value 'HKCU:\SOFTWARE\Microsoft\Office\16.0' -Option Constant
Set-Variable -Name DOMAINS_REGISTRY -Value 'HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMap\Domains' -Option Constant
Set-Variable -Name SERVER_PATH -Value '\\fs1.concept.ru\' -Option Constant
Set-Variable -Name TEMPLATES_PATH -Value "$Env:APPDATA\Microsoft\Templates" -Option Constant

$word = If (Test-Path "${OFFICE_REGISTRY}\Word") {"${OFFICE_REGISTRY}\Word"} Else {''}
$excel = If (Test-Path "${OFFICE_REGISTRY}\Excel") {"${OFFICE_REGISTRY}\Excel"} Else {''}
$visio = If (Test-Path "${OFFICE_REGISTRY}\Visio") {"${OFFICE_REGISTRY}\Visio"} Else {''}

$desktopPath = [Environment]::GetFolderPath('Desktop')
$localHome = 'C:\Tools'
$Env:PSModulePath += ";${localHome}"

function InstallAllProducts {
    [CmdletBinding()]
    Param(
    [Parameter(Mandatory=$true)]
        [string] $distrPath,
        [switch] $skipShortcuts = $false,
        [switch] $standalone = $false
    )
    
    Set-StrictMode -Version 3.0

    if (-not (Test-Path -Path ${distrPath})) {
        Write-Error "Distribution path does not exist: ${distrPath}"
        Exit 1
    }

    $InformationPreference="Continue"

    StartLog

    Write-Host "Starting installation procedure for Concept technologies" -ForegroundColor DarkGreen
    Write-Host "Sources: ${distrPath}"
    Write-Host "Skip shortcuts: ${skipShortcuts}`n"

    Write-Host "Dependencies status:" -ForegroundColor DarkGreen
    StateSystemStatus

    $driveLetter = MountPathDrive $distrPath
    $distr = "${driveLetter}`:"

    Write-Host "`nEnsure core is updated..." -ForegroundColor DarkGreen
    PrepareCore $distr $distrPath $standalone $localHome

    Write-Host "`nSetup SecureLocation options for ${SERVER_PATH}" -ForegroundColor DarkGreen
    SetupTrustedLocations $SERVER_PATH

    Write-Host "`nInitializing template path for Office applications: ${TEMPLATES_PATH}" -ForegroundColor DarkGreen
    InitializeTemplates $TEMPLATES_PATH

    Write-Host "Update templates at ${TEMPLATES_PATH}" -ForegroundColor DarkGreen
    UpdateTemplates "${distr}\data\Templates" $TEMPLATES_PATH $localHome

    Write-Host "`nUpdating products..." -ForegroundColor DarkGreen
    UpdateProducts $distr $TEMPLATES_PATH $localHome

    if (-not ($skipShortcuts)) {
        Write-Host "`nInstalling desktop shortcuts" -ForegroundColor DarkGreen
        Copy-Item -Path "${distr}\data\Shortcuts\*" `
            -Destination $desktopPath `
            -Exclude (Get-ChildItem $desktopPath) -Force -Recurse
    }

    Write-Host "`nClean up procedure ..." -ForegroundColor Blue
    Remove-PSDrive -Name $driveLetter
    
    Write-Host "`nInstallation complete" -ForegroundColor Green

    StopLog
}

function StartLog {
    $ErrorActionPreference='SilentlyContinue'
    Stop-Transcript | Out-Null
    $ErrorActionPreference = "Continue"

    $dateSuffix = (Get-Date -UFormat "%Y%m%d_%I-%M-%S").ToString()
    $logFile = "${PSScriptRoot}\logs\${dateSuffix}_$($Env:USERNAME).txt"
    Start-Transcript -Path $logFile -IncludeInvocationHeader
}

function StopLog {
    Stop-Transcript
}

function CheckProcess($processName) {
    $app = Get-Process $processName -ErrorAction SilentlyContinue
    return -not ($null -eq $app)
}

function StateSystemStatus() {
    if ($word) {
        Write-Host "Word running: $(CheckProcess 'WINWORD')"
    } else {
        Write-Host 'Word not found' -ForegroundColor DarkRed
    }
    if ($excel) {
        Write-Host "Excel running: $(CheckProcess 'EXCEL')"
    } else {
        Write-Host 'Excel not found' -ForegroundColor DarkRed
    }
    if ($visio) {
        Write-Host "Visio running: $(CheckProcess 'VISIO')"
    } else {
        Write-Host 'Visio not found' -ForegroundColor DarkRed
    }
}

function MountPathDrive($targetPath) {
    Write-Host "`nCreating temporary drive for ${targetPath}" -ForegroundColor DarkGreen
    [char[]] $takenDrive = (Get-PSDrive -Name [E-Z]).Name
    $driveLetter = ([char[]] (69..90)).Where({ $_ -notin $takenDrive }, 'First')[0]
    if (!$driveLetter) {
        Write-Error "No available drive letter"
        Exit 1
    }
    New-PSDrive -Name $driveLetter -PSProvider FileSystem -Root $targetPath -Scope Global | Out-Null
    Write-Host "Drive ${driveLetter} mounted"
    return $driveLetter
}

function UpdateTemplates($source, $destination, $localHome) {
    Copy-Item -Path "${source}\*" -Destination "$destination\" -Force -Recurse

    $templatesFile = "${localHome}\script\templates_list.txt"
    $sourcePrefix = $(Resolve-Path $source).ProviderPath
    Write-Host "Save temapltes list to $templatesFile"
    Get-ChildItem -Recurse $source | Where-Object { ! $_.PSIsContainer } | Select-Object FullName `
        | ForEach-Object { $_.FullName.Replace("$sourcePrefix\",'') } `
        | Out-File -Encoding "UTF8" $templatesFile
}

function UpdateProducts($distr, $templates, $localHome) {
    $serverManifest = "${distr}\distribution_manifest.json"
    $serverJSON = Open-DistrManifest $serverManifest
    
    $localManifest = "${localHome}\distribution_manifest.json"
    $localJSON = Open-DistrManifest $localManifest
    $firstRun = $localJSON.Count -eq 0

    Write-Host "`n"
    foreach ($product in $serverJSON.Keys) {
        $serverVersion = [System.Version] $serverJSON.$product
        if ($localJSON.Contains($product)) {
            $localVersion = [System.Version] $localJSON.$product
        } else {
            $localVersion = [System.Version] "0.0.0"
        }
        if ($serverVersion -eq $localVersion) {
            Write-Host "Product ${product}: ${serverVersion} ... up to date`n" -ForegroundColor Gray
        } elseif (Install-Product $product $distr $templates $localHome) {
            Write-Host "Product ${product}: ${serverVersion} ... updated from ${localVersion}`n" -ForegroundColor DarkGreen
            $localJSON[$product] = $serverJSON.$product
        } else {
            Write-Host "Product ${product}: ${serverVersion} ... failed to update from ${localVersion}`n" -ForegroundColor DarkRed
        }
    }
    if ($firstRun) {
        Write-Host "`nFirst time run. Enabling default addins" -ForegroundColor DarkGreen
        Enable-Product 'WordHelper' $localHome | Out-Null
        Enable-Product 'ExcelHelper' $localHome | Out-Null
        Enable-Product 'Concept-Reports' $localHome | Out-Null
    }

    Save-DistrManifest $localJSON $localManifest
}

function PrepareCore($distrMount, $distrPath, $standalone, $localHome) {
    if (CheckConceptDeploy $distrMount) {
        Import-Module -Name ConceptDeploy
        Write-Host "ConceptDeploy already installed: $((Get-Module -Name 'ConceptDeploy').Version)" -ForegroundColor Gray
    } else {
        Write-Host "Installing Powershell library: ConceptDeploy" -ForegroundColor DarkGreen
        . "$distrMount\src\ConceptDeploy\CD_Install.ps1"
        Import-Module -Name ConceptDeploy
        Write-Host "Installed version: ConceptDeploy $((Get-Module -Name 'ConceptDeploy').Version)"
    }

    if (-not (Test-Path -Path "${localHome}\script\")) {
        New-Item -Path "${localHome}\script\" -ItemType Directory -Force | Out-Null
    }
    Copy-Item -Path "${distrMount}\UpdateConceptProducts.ps1" -Destination "${localHome}\script\"
    Copy-Item -Path "${distrMount}\UninstallConceptProducts.ps1" -Destination "${localHome}\script\"
    Copy-Item -Path "${distrMount}\uninstall.bat" -Destination "${localHome}\script\uninstall-concept.bat"
    Write-Host "Installed scripts to $localHome\script\" -ForegroundColor Gray

    Write-Host "`nEnsure Python environement is updated..." -ForegroundColor DarkGreen

    $python = ''
    if ($standalone) {
        if ([Environment]::Is64BitProcess) {
            $python = Initialize-Python -PythonDistr "${distrPath}\distr\python-3.12.2-amd64.exe"
        } else {
            Write-Host "32-bit system do not support Standalone installation mode" -ForegroundColor Red
        }
    } else {
        $python = Initialize-Python
    }

    if ($python -eq '') {
        Write-Host "Skip python libraries because could not initialize python" -ForegroundColor DarkRed
    } elseif ($standalone) {
        $pythonFolder = (Get-item $python).Directory.Parent.FullName
        Expand-Archive -Path "${distrMount}\distr\Python312-venv.zip" -DestinationPath $pythonFolder -Force
    } else {
        Install-PythonLibs -python $python -localDistr "${distrPath}\src"
    }

    Write-Host "`nUpdate dynamic libraries at ${localHome}" -ForegroundColor DarkGreen
    if (-not (Test-Path -Path "${localHome}\dll")) {
        New-Item -Path "${localHome}\dll" -ItemType Directory -Force | Out-Null
    }
    Copy-Item -Path "${distrMount}\dll" -Destination "${localHome}\" -Force -Recurse

    $conceptLocal = Get-ConceptLocal
    if (-not (Test-Path -Path "${conceptLocal}\models")) {
        New-Item -Path "${conceptLocal}\models" -ItemType Directory -Force | Out-Null
    }
    if (-not (Test-Path -Path "${localHome}\optional")) {
        New-Item -Path "${localHome}\optional" -ItemType Directory -Force | Out-Null
    }
    if (-not (Test-Path -Path "${localHome}\models")) {
        New-Item -Path "${localHome}\models" -ItemType Directory -Force | Out-Null
    }
}

function CheckConceptDeploy($distr) {
    $currentModule = Get-Module -ListAvailable ConceptDeploy
    if (!$currentModule) {
        return $false
    }
    $newVersion = [System.Version] (Get-Content -Path "${distr}\src\ConceptDeploy\VERSION")
    $oldVersion = $currentModule.Version
    return $oldVersion -ge $newVersion
}

function InitializeTemplates($templates) {
    $eap = $ErrorActionPreference
    $ErrorActionPreference = 'SilentlyContinue'

    if (-not (Test-Path -Path $templates)) {
        New-Item -ItemType Directory -Path $templates -Force | Out-Null
        Write-Host "Created office templates directory: ${templates}"
    }

    if ($word) {
        if(-not (
            Get-ItemProperty "${word}\Options" |
            ForEach-Object { if ($_ -like '*PersonalTemplates*') { return $true } }
        )) {
            New-ItemProperty -Path "${word}\Options" -Name 'PersonalTemplates' -Value $templates -PropertyType ExpandString -Force
        } else {
            Set-ItemProperty -Path "${word}\Options" -Name 'PersonalTemplates' -Value $templates
        }
    }
    if ($excel) {
        if(-not (
            Get-ItemProperty "${excel}\Options" |
            ForEach-Object { if ($_ -like '*PersonalTemplates*') { return $true } }
        )) {
            New-ItemProperty -Path "${excel}\Options" -Name 'PersonalTemplates' -Value $templates -PropertyType ExpandString -Force
        } else {
            Set-ItemProperty -Path "${excel}\Options" -Name 'PersonalTemplates' -Value $templates
        }
    }
    if ($visio) {
        if(-not (
            Get-ItemProperty "${visio}\Application" | 
            ForEach-Object { if ($_ -like '*PersonalTemplates*') { return $true } }
        )) {
            New-ItemProperty -Path "${visio}\Options" -Name 'PersonalTemplates' -Value $templates -PropertyType ExpandString -Force
        } else {
            Set-ItemProperty -Path "${visio}\Application" -Name 'PersonalTemplates' -Value $templates
        }
    }
    $ErrorActionPreference = $eap
}

function SetupTrustedLocations($serverLocation) {
    if ($word) {
        if (AddTrustedLocation $word $serverLocation) {
            Write-Host 'Word ... DONE'
        } else {
            Write-Host 'Word ... EXISTS'
        }
    }
    if ($excel) {
        if (AddTrustedLocation $excel $serverLocation) {
            Write-Host 'Excel ... DONE'
        } else {
            Write-Host 'Excel ... EXISTS'
        }
    }
    if ($visio) {
        if (AddTrustedLocation $visio $serverLocation) {
            Write-Host 'Visio ... DONE'
        } else {
            Write-Host 'Visio ... EXISTS'
        }
    }
    $conceptDomain = "${DOMAINS_REGISTRY}\concept.ru\fs1"
    if (Test-Path -Path $conceptDomain) {
        Write-Host 'Intranet settings ... EXISTS'
    } else {
        New-Item -Path "${DOMAINS_REGISTRY}\concept.ru" -ItemType Directory | Out-Null
        New-Item -Path $conceptDomain -ItemType Directory | Out-Null
        New-ItemProperty -Path $conceptDomain -Name 'file' -Value 1 | Out-Null
        Write-Host 'Intranet settings ... DONE'
    }
}

function AddTrustedLocation($target, $location) {
    $registryPath = "${target}\Security\Trusted Locations"
    if (-not(Test-Path -Path $registryPath)) {
        if (-not(Test-Path -Path "${target}\Security")){
            New-Item -Path "${target}\Security" -ItemType Directory | Out-Null
        }
        New-Item -Path $registryPath -ItemType Directory | Out-Null
    }
    Set-ItemProperty -Path $registryPath -Name 'AllowNetworkLocations' -Value 1
    if (
        Get-ChildItem -Path $registryPath -Recurse |
        ForEach-Object { if ($_.GetValue('Path') -eq $location) { return $true } }
    ) {
        return $false
    } else {
        $index = 3
        while (Test-Path -Path "${registryPath}\Location${index}") { $index++ }
        $folder = "${registryPath}\Location${index}"
        New-Item -Path $folder -ItemType Directory | Out-Null
        New-ItemProperty -Path $folder -Name 'Path' -Value $location | Out-Null
        New-ItemProperty -Path $folder -Name 'Description' -Value 'CIHT Concept Network' | Out-Null
        New-ItemProperty -Path $folder -Name 'AllowSubfolders' -Value 1 | Out-Null
        return $true
    }
}

# InstallAllProducts "D:\test\ConceptDistr"
# SetupTrustedLocations \\fs1.concept.ru\