# Install Concept products from local distr


$localHome = 'C:\Tools'
$Env:PSModulePath += ";${localHome}"

function UpdateConceptProducts {
    [CmdletBinding()]
    Param(
    [Parameter(Mandatory=$true)]
        [string] $distrPath,
        [string[]] $products = $(, 'all')
    )

    Set-StrictMode -Version 3.0

    if ($products.Count -lt 1) {
        Write-Host 'Missing products list'
        Exit 1
    }

    StartLog

    Write-Host "Starting update procedure for Concept technologies" -ForegroundColor DarkGreen
    Write-Host "Sources: ${distrPath}"
    Write-Host "Requested products: ${products}"

    $updates = New-Object System.Collections.Generic.List[System.Object]
    $enable_products = New-Object System.Collections.Generic.List[System.Object]
    $disable_products = New-Object System.Collections.Generic.List[System.Object]
    if ($products[0] -eq 'all') {
        $updates.Add('all')
    } else {
        foreach ($p in $products) {
            if ($p.EndsWith('-') -or $p.EndsWith('+')) {
                $product_id = $p.Substring(0, $p.Length - 1)
            } else {
                $product_id = $p
            }
            if ($p.EndsWith('-')) {
                $disable_products.Add($product_id)
            } else {
                $updates.Add($product_id)
                $enable_products.Add($product_id)
            }
        }
    }
    Write-Host "Check updates: $updates"
    Write-Host "Enable products: $enable_products"
    Write-Host "Disable products: $disable_products"

    if (Test-Path -Path "filesystem::$distrPath\distribution_manifest.json") {
        Write-Host "Check path $distrPath\distribution_manifest.json ... ACCESS GRANTED" -ForegroundColor DarkGreen
        UpdateProducts $distrPath $updates
        
    } else {
        Write-Host "Check path $distrPath\distribution_manifest.json ... NO ACCESS" -ForegroundColor DarkRed
        Write-Host "Skipping product updates..." -ForegroundColor DarkRed
    }
    
    $tools = 'C:\Tools'
    foreach ($product in $enable_products) {
        if (Enable-Product $product $tools) {
            Write-Host "Product $($product): enabled`n" -ForegroundColor DarkGreen
        } else {
            Write-Host "Product $($product): failed to enable`n" -ForegroundColor DarkRed
        }
    }

    $tools = 'C:\Tools'
    foreach ($product in $disable_products) {
        if (Disable-Product $product) {
            Write-Host "Product $($product): disabled`n" -ForegroundColor DarkGreen
        } else {
            Write-Host "Product $($product): failed to disable`n" -ForegroundColor DarkRed
        }
    }
    
    Write-Host "`nUpdate complete" -ForegroundColor Green

    StopLog
}

function StartLog {
    $ErrorActionPreference="SilentlyContinue"
    Stop-Transcript | Out-Null
    $ErrorActionPreference = "Continue"

    $dateSuffix = (Get-Date -UFormat "%Y%m%d_%I-%M-%S").ToString()
    $logFile = "$PSScriptRoot\logs\$($dateSuffix)_CP-update_$($Env:USERNAME).txt"
    Start-Transcript -Path $logFile -IncludeInvocationHeader
}

function StopLog {
    Stop-Transcript
}

function UpdateProducts($distrPath, $products) {
    $driveLetter = MountPathDrive $distrPath
    $distr = "$driveLetter`:"

    if (-not(CheckConceptDeploy $distr)) {
        Write-Host "Installing Powershell library: ConceptDeploy" -ForegroundColor DarkGreen
        . "$driveLetter\src\ConceptDeploy\CD_Install.ps1"
    }
    
    Import-Module -Name ConceptDeploy
    Write-Host "Using ConceptDeploy: $((Get-Module -Name 'ConceptDeploy').Version)" -ForegroundColor Gray

    if ($products.Count -ne 0) {
        $serverManifest = "$distr\distribution_manifest.json"
        $serverJSON = Open-DistrManifest $serverManifest

        $localManifest = "C:\Tools\distribution_manifest.json"
        $localJSON = Open-DistrManifest $localManifest
        
        if ($products[0] -eq 'all') {
            $products = $serverJSON.Keys
        }
        foreach ($product in $products) {
            $serverVersion = [System.Version] $serverJSON.$product
            if ($localJSON.Contains($product)) {
                $localVersion = [System.Version] $localJSON.$product
            } else {
                $localVersion = [System.Version] "0.0.0"
            }
            if ($serverVersion -eq $localVersion) {
                Write-Host "Product $($product): $serverVersion ... up to date`n" -ForegroundColor Gray
            } elseif (Install-Product $product $distr $templates $localTools) {
                Write-Host "Product $($product): $serverVersion ... updated from $localVersion`n" -ForegroundColor DarkGreen
                $localJSON[$product] = $serverJSON.$product
            } else {
                Write-Host "Product $($product): $serverVersion ... failed to update from $localVersion`n" -ForegroundColor DarkRed
            }
        }
        Save-DistrManifest $localJSON $localManifest
    }
    
    Write-Host "`nUnmount $driveLetter" -ForegroundColor DarkGreen
    Remove-PSDrive -Name $driveLetter
}

function MountPathDrive($targetPath) {
    Write-Host "`nCreating temporary drive for $targetPath" -ForegroundColor DarkGreen
    [char[]] $takenDrive = (Get-PSDrive -Name [E-Z]).Name
    $driveLetter = ([char[]] (69..90)).Where({ $_ -notin $takenDrive }, 'First')[0]
    if (!$driveLetter) {
        Write-Error "No available drive letter"
        Exit 1
    }
    New-PSDrive -Name $driveLetter -PSProvider FileSystem -Root $targetPath -Scope Global | Out-Null
    Write-Host "Drive $driveLetter mounted"
    return $driveLetter
}

function CheckConceptDeploy($distr) {
    $currentModule = Get-Module -ListAvailable ConceptDeploy
    if (!$currentModule) {
        return $false
    }
    $newVersion = [System.Version] (Get-Content -Path "$distr\src\ConceptDeploy\VERSION")
    $oldVersion = $currentModule.Version
    return $oldVersion -ge $newVersion
}


# UpdateConceptProducts -DistrPath '\\fs1.concept.ru\Exchange\ConceptDistr' 'Concept-Maket-', 'Concept-Mining+'